@@ -19,10 +19,10 @@
 
 
 void chomp(QCString *line);
-QString quote(QString str);
-QString joinLine(QStrList list);
-QStrList splitLine(QString line, char delim=' ');
-bool isValidTag(QString str);
+//QString quote(const QString &str);
+QString joinLine(const QStringList &list);
+QStringList splitLine(QString, char delim=' ');
+bool isValidTag(const QString &str);
 QString cvsClient();
 QString userName();
 
