@@ -29,12 +29,12 @@ class TagDialog : public QDialog
 public:
     enum ActionType { Create, Delete };
     
-    TagDialog( ActionType action, QString sbox, QString repo,
+    TagDialog( ActionType action, const QString &sbox, const QString &repo,
                QWidget *parent=0, const char *name=0 );
     
-    bool branchTag()
+    bool branchTag() const
         { return branchtag_button->isChecked(); }
-    QString tag()
+    QString tag() const
         { return act==Delete? tag_combo->currentText() : tag_edit->text(); }
 
 protected:
