@@ -34,6 +34,9 @@
 #include <klocale.h>
 #include <kmenubar.h>
 #include <ktoolbarbutton.h>
+#include <kprocess.h>
+#include <kfileitem.h>
+#include <kdebug.h>
 
 #include "logdlg.h"
 #include "loglist.h"
@@ -59,6 +62,8 @@
 #include "toplevel.moc"
 
 
+#define COMMIT_SPLIT_CHAR '\r'
+
 TopLevel::TopLevel(const char *name)
     : KMainWindow(0, name )
 {
@@ -106,21 +111,21 @@ void TopLevel::setupAccel()
     accel = new KAccel(this);
 
     accel->insertItem( i18n("Open File"),
-		       "Open", CTRL+Key_O );
+                       "Open", CTRL+Key_O );
     accel->insertItem( i18n("Update Files"),
-		       "Update", CTRL+Key_U );
+                       "Update", CTRL+Key_U );
     accel->insertItem( i18n("Status"),
                        "Status", Key_F5 );
     accel->insertItem( i18n("Commit file"),
-		       "Commit", Key_NumberSign );
+                       "Commit", Key_NumberSign );
     accel->insertItem( i18n("Add File to Repository"),
-		       "Add", Key_Plus );
+                       "Add", Key_Plus );
     accel->insertItem( i18n("Remove File from Repository"),
-		       "Remove", Key_Minus );
+                       "Remove", Key_Minus );
     accel->insertItem( i18n("Browse log file"),
-		       "Log", CTRL+Key_L );
+                       "Log", CTRL+Key_L );
     accel->insertItem( i18n("View Annotate"),
-		       "Annotate", CTRL+Key_A );
+                       "Annotate", CTRL+Key_A );
     accel->insertItem( i18n("Diff against repository"),
                        "Diff", CTRL+Key_D );
     accel->insertItem( i18n("Hide all files"),
@@ -179,9 +184,9 @@ void TopLevel::setupMenuBar()
     file->insertSeparator();
     
     file_update = file->insertItem( i18n("&Update"),
-				    this, SLOT(slotUpdate()) );
+                                    this, SLOT(slotUpdate()) );
     file_status = file->insertItem( i18n("&Status"),
-				    this, SLOT(slotStatus()) );
+                                    this, SLOT(slotStatus()) );
 
     file->insertSeparator();
     
@@ -193,20 +198,20 @@ void TopLevel::setupMenuBar()
     file->insertSeparator();
     
     file_commit = file->insertItem( i18n("&Commit..."),
-				    this, SLOT(slotCommit()) );
+                                    this, SLOT(slotCommit()) );
     file_add    = file->insertItem( i18n("&Add to Repository..."),
-				    this, SLOT(slotAdd()) );
+                                    this, SLOT(slotAdd()) );
     file_addbin = file->insertItem( i18n("Add &binary..."),
                                     this, SLOT(slotAddBinary()) );
     file_remove = file->insertItem( i18n("&Remove from Repository..."),
-				    this, SLOT(slotRemove()) );
+                                    this, SLOT(slotRemove()) );
     file_revert = file->insertItem( i18n("Rever&t"),
                                     this, SLOT(slotRevert()) );
 
     file->insertSeparator();
 
     file_exit   = file->insertItem( BarIcon("exit.xpm"), i18n("&Quit"),
-				    this, SLOT(slotExit()) );
+                                    this, SLOT(slotExit()) );
 
     //
     // View menu
@@ -214,15 +219,15 @@ void TopLevel::setupMenuBar()
     view = new QPopupMenu();
     
     view_log    = view->insertItem( i18n("Browse &Log..."),
-				    this, SLOT(slotBrowseLog()) );
+                                    this, SLOT(slotBrowseLog()) );
 #if 0
     view_mlog   = view->insertItem( i18n("Browse Multi-File Log..."),
-				    this, SLOT(slotBrowseMultiLog()) );
+                                    this, SLOT(slotBrowseMultiLog()) );
 #endif
     view_anno   = view->insertItem( i18n("&Annotate..."),
-				    this, SLOT(slotAnnotate()) );
+                                    this, SLOT(slotAnnotate()) );
     view_diff   = view->insertItem( i18n("&Difference to Repository..."),
-				    this, SLOT(slotDiff()) );
+                                    this, SLOT(slotDiff()) );
     view_change = view->insertItem( i18n("Last &Change..."),
                                     this, SLOT(slotLastChange()) );
     view_history= view->insertItem( i18n("&History..."),
@@ -232,9 +237,9 @@ void TopLevel::setupMenuBar()
     view->insertSeparator();
 
     view_unfold = view->insertItem( i18n("&Unfold file tree"),
-				    update, SLOT(unfoldTree()) );
+                                    update, SLOT(unfoldTree()) );
     view_fold =   view->insertItem( i18n("&Fold file tree"),
-				    update, SLOT(foldTree()) );
+                                    update, SLOT(foldTree()) );
 
     //
     // Advanced menu
@@ -288,7 +293,7 @@ void TopLevel::setupMenuBar()
     repo = new QPopupMenu();
 
     repo_checkout = repo->insertItem( i18n("&Checkout..."),
-				      this, SLOT(slotCheckout()) );
+                                        this, SLOT(slotCheckout()) );
     repo_import   = repo->insertItem( i18n("&Import..."),
                                       this, SLOT(slotImport()) );
 
@@ -307,7 +312,7 @@ void TopLevel::setupMenuBar()
     settings_hideuptodate= settings->insertItem( i18n("Hide up-&to-date files"),
                                                  this, SLOT(slotHideUpToDate()) );
     settings_hideremoved = settings->insertItem( i18n("Hide removed files"),
-						 this, SLOT(slotHideRemoved()) );
+                                                 this, SLOT(slotHideRemoved()) );
 
     settings->insertSeparator();
     
@@ -376,22 +381,22 @@ void TopLevel::setupToolBar()
     KToolBar *toolbar = toolBar();
 
     toolbar->insertButton( BarIcon("up.xpm"), 101, SIGNAL(clicked()),
-			   this, SLOT(slotCommit()),
-			   true, i18n("Commit") );
+                           this, SLOT(slotCommit()),
+                           true, i18n("Commit") );
 
     toolbar->insertButton( BarIcon("down.xpm"), 102, SIGNAL(clicked()),
-			   this, SLOT(slotUpdate()),
-			   true, i18n("Update") );
+                           this, SLOT(slotUpdate()),
+                           true, i18n("Update") );
 
     toolbar->insertSeparator();
     
     toolbar->insertButton( BarIcon("fileopen.xpm"), 103, SIGNAL(clicked()),
-			   this, SLOT(slotOpen()),
-			   true, i18n("Open file") );
+                           this, SLOT(slotOpen()),
+                           true, i18n("Open file") );
     
     toolbar->insertButton( BarIcon("help.xpm"), 104, SIGNAL(clicked()),
-			   this, SLOT(slotHelp()),
-			   true, i18n("Help") );
+                           this, SLOT(slotHelp()),
+                           true, i18n("Help") );
 
     toolbar->insertSeparator();
     
@@ -407,8 +412,8 @@ void TopLevel::setupStatusBar()
     filterLabel->setFixedSize(filterLabel->sizeHint());
     filterLabel->setText("");
     QToolTip::add(filterLabel, i18n("F - All files are hidden, the tree shows only directories\n"
-				    "N - All up-to-date files are hidden\n"
-				    "R - All removed files are hidden"));
+                                    "N - All up-to-date files are hidden\n"
+                                    "R - All removed files are hidden"));
     statusBar()->addWidget(filterLabel, 0, true);
 }
 
@@ -481,9 +486,11 @@ void TopLevel::popupRequested()
                                     this, SLOT(slotAdd()) );
     int id_remove = pop.insertItem( i18n("&Remove from Repository..."),
                                     this, SLOT(slotRemove()) );
-    bool single = update->hasSingleSelection();
-    pop.setItemEnabled(id_open, single);
-    pop.setItemEnabled(id_resolv, single);
+
+    int numselected = update->numSelections();
+    pop.setItemEnabled(id_open, (numselected >= 1));
+    pop.setItemEnabled(id_resolv, (numselected == 1));
+
     bool nojob = !toolBar()->getButton(42)->isEnabled();
     pop.setItemEnabled(id_commit, nojob);
     pop.setItemEnabled(id_add, nojob);
@@ -494,9 +501,13 @@ void TopLevel::popupRequested()
 
 void TopLevel::updateMenu()
 {
-    bool single = update->hasSingleSelection();
+    int num = update->numSelections();
+    bool single = false;
+    if( num == 1 ) single = true;
+
     toolBar()->getButton(103)->setEnabled(single);
-    file->setItemEnabled(file_open, single);
+    file->setItemEnabled(file_open, (num >= 1) );
+
     file->setItemEnabled(file_resolv, single);
     view->setItemEnabled(view_log, single);
     view->setItemEnabled(view_anno, single);
@@ -521,6 +532,11 @@ void TopLevel::updateMenu()
     advanced->setItemEnabled(advanced_addwatch, nojob);
     advanced->setItemEnabled(advanced_remwatch, nojob);
     advanced->setItemEnabled(advanced_watchers, nojob);
+    advanced->setItemEnabled(advanced_edit, nojob);
+    advanced->setItemEnabled(advanced_unedit, nojob);
+    advanced->setItemEnabled(advanced_editors, nojob);
+    advanced->setItemEnabled(advanced_lock, nojob);
+    advanced->setItemEnabled(advanced_unlock, nojob);
     repo->setItemEnabled(repo_checkout, nojob);
     repo->setItemEnabled(repo_import, nojob);
 }
@@ -571,62 +587,73 @@ void TopLevel::slotChangeLog()
 
 void TopLevel::slotOpen()
 {
-    QString filename;
-    update->getSingleSelection(&filename);
-    if (!filename.isEmpty())
-	openFile(filename);
+    QStringList filenames = update->multipleSelection();
+    openFiles(filenames);
 }
 
 
 void TopLevel::openFile(QString filename)
 {
-    if (opt_doCVSEdit && !QFileInfo(filename).isWritable())
+    QStringList files;
+    files << filename;
+    openFiles( files );
+}
+
+
+void TopLevel::openFiles(const QStringList &filenames)
+{
+    // First check the cvs edit stuff
+    if (opt_doCVSEdit) // && !QFileInfo(filename).isWritable())
         {
             CvsProgressDialog l("Edit", this);
             l.setCaption("CVS Edit");
             QString cmdline = cvsClient() + " edit ";
-            cmdline += quote(filename);
-            if (!l.execCommand(sandbox, repository, cmdline, "edit") ||
-                !QFileInfo(filename).isWritable())
-                return;
+
+            bool doit = false;
+            for ( QStringList::ConstIterator it = filenames.begin();
+                  it != filenames.end(); ++it )
+                {
+                    if (!QFileInfo(*it).isWritable())
+                        {
+                            doit = true;
+                            break;
+                        }
+                    
+                    cmdline += " ";
+                    cmdline += KShellProcess::quote(*it);
+                }
+            
+            if (doit)
+                if (!l.execCommand(sandbox, repository, cmdline, "edit"))
+                    return;
         }
     
+    // Now open the files by starting a new instance of kwrite...
     KConfig *config = kapp->config();
     config->setGroup("Communication");
-#if 0
-    bool usedcop = config->readBoolEntry("UseDCOP", false);
-    QCString client = config->readEntry("DCOPClient", "KDevelop").latin1();
-    QCString object = config->readEntry("DCOPObject", "Application").latin1();
-    if (usedcop)
+    KShellProcess proc("/bin/sh");
+    proc << config->readEntry("Editor", "kwrite");
+
+    QString path;
+    bool run = false;
+    for ( QStringList::ConstIterator it = filenames.begin();
+          it != filenames.end(); ++it )
+    {
+        //make sure that the file is a file, not a directory
+        path = sandbox + "/" + *it;
+        kdDebug() << "examing: " << path << endl;
+
+        KFileItem info( -1, -1, KURL( path ) );
+        if( !info.isDir() )
         {
-            kapp->dcopClient()->attach();
-            kapp->dcopClient()->registerAs("cervisia");
-            if (!kapp->dcopClient()->isApplicationRegistered(client))
-                usedcop = false;
+            kdDebug() << "file is not a dir" << endl;
+            run = true;
+            proc << KShellProcess::quote( path );
         }
-    if (usedcop)
-        {
-            QByteArray params;
-            QDataStream stream(params, IO_WriteOnly);
-            stream << QFileInfo(filename).absFilePath();
-            capp->dcopClient()->send(client, object,
-                                     "openDocument(QString)", params);
-        }
-    else
-        {
-            QString command = config->readEntry("Editor", "kwrite");
-            command += " ";
-            command += quote(filename);
-            command += "&";
-            ::system(command);
-        }
-#else
-    QString command = config->readEntry("Editor", "kwrite");
-    command += " ";
-    command += quote(filename);
-    command += "&";
-    ::system(command);
-#endif
+    }
+
+    if( run )
+        proc.start(KProcess::DontCare);
 }
 
 
@@ -673,7 +700,7 @@ void TopLevel::slotUpdateToTag()
             else
                 {
                     tagopt = "-D ";
-                    tagopt += quote(l->date());
+                    tagopt += KShellProcess::quote(l->date());
                 }
             tagopt += " ";
             updateOrStatus(false, tagopt);
@@ -744,9 +771,9 @@ void TopLevel::slotRemove()
 }
 
 
-void TopLevel::updateOrStatus(bool noact, QString extraopt)
+void TopLevel::updateOrStatus(bool noact, const QString &extraopt)
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
     
@@ -782,69 +809,82 @@ void TopLevel::updateOrStatus(bool noact, QString extraopt)
 
 void TopLevel::commitOrAddOrRemove(CommitDialog::ActionType action)
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
     
     // modal dialog
     CommitDialog *l = new CommitDialog(action, this);
     if (action == CommitDialog::Commit)
-        {
-            l->setLogMessage(changelogstr);
-            l->setLogHistory(sandbox, repository, recentCommits);
-        }
+    {
+        l->setLogMessage(changelogstr);
+        l->setLogHistory(sandbox, repository, recentCommits);
+    }
     l->setFileList(list);
     
     if (l->exec())
+    {
+        QString cmdline;
+        switch (action)
         {
-            QString cmdline;
-	    switch (action)
-		{
-		case CommitDialog::Commit:
-		    {
-                        recentCommits.insert(0, l->logMessage());
-                        while (recentCommits.count() > 30)
-                            recentCommits.removeLast();
-                        update->prepareJob(opt_commitRecursive, UpdateView::Commit);
-			cmdline = cvsClient() + " commit ";
-                        if (opt_commitRecursive)
-                            cmdline += "-R ";
-                        else
-                            cmdline += "-l ";
-			cmdline += "-m ";
-			cmdline += quote(l->logMessage());
-			cmdline += " ";
-		    }
-		    break;
-		case CommitDialog::Add:
-                    update->prepareJob(false, UpdateView::Add);
-		    cmdline = cvsClient() + " add ";
-		    break;
-		case CommitDialog::AddBinary:
-                    update->prepareJob(false, UpdateView::Add);
-		    cmdline = cvsClient() + " add -kb ";
-		    break;
-		case CommitDialog::Remove:
-                    update->prepareJob(opt_commitRecursive, UpdateView::Remove);
-		    cmdline = cvsClient() + " remove -f ";
-                    if (opt_commitRecursive)
-                        cmdline += "-R ";
-                    else
-                        cmdline += "-l ";
-		    break;
-		}
-	    cmdline += joinLine(list);
-            cmdline += " 2>&1";
-            
-            if (protocol->startJob(sandbox, repository, cmdline))
+            case CommitDialog::Commit:
+            {
+                QString msg = l->logMessage();
+                if( !recentCommits.contains( msg ) )
                 {
-                    showJobStart(cmdline);
-                    connect( protocol, SIGNAL(jobFinished(bool)),
-                             update, SLOT(finishJob(bool)) );
-                    connect( protocol, SIGNAL(jobFinished(bool)),
-                             this, SLOT(slotJobFinished(bool)) );
+                    recentCommits.prepend( msg );
+                    while (recentCommits.count() > 50)
+                        recentCommits.remove( recentCommits.last() );
+
+                    KConfig* config = kapp->config();
+                    config->setGroup( "CommitLogs" );
+                    config->writeEntry( sandbox, recentCommits, COMMIT_SPLIT_CHAR );
                 }
+
+                update->prepareJob(opt_commitRecursive, UpdateView::Commit);
+                cmdline = cvsClient() + " commit ";
+                if (opt_commitRecursive)
+                    cmdline += "-R ";
+                else
+                    cmdline += "-l ";
+                cmdline += "-m ";
+                cmdline += KShellProcess::quote(l->logMessage());
+                cmdline += " ";
+            }
+            break;
+
+            case CommitDialog::Add:
+                update->prepareJob(false, UpdateView::Add);
+                cmdline = cvsClient() + " add ";
+            break;
+
+            case CommitDialog::AddBinary:
+                update->prepareJob(false, UpdateView::Add);
+                cmdline = cvsClient() + " add -kb ";
+            break;
+
+            case CommitDialog::Remove:
+                update->prepareJob(opt_commitRecursive, UpdateView::Remove);
+                cmdline = cvsClient() + " remove -f ";
+                if (opt_commitRecursive)
+                    cmdline += "-R ";
+                else
+                    cmdline += "-l ";
+            break;
         }
+
+        cmdline += joinLine(list);
+        cmdline += " 2>&1";
+            
+        if (protocol->startJob(sandbox, repository, cmdline))
+        {
+            showJobStart(cmdline);
+            connect( protocol, SIGNAL(jobFinished(bool)),
+                     update, SLOT(finishJob(bool)) );
+            connect( protocol, SIGNAL(jobFinished(bool)),
+                     this, SLOT(slotJobFinished(bool)) );
+        }
+    }
     
     delete l;
 }
@@ -878,14 +918,14 @@ void TopLevel::slotBrowseMultiLog()
 {
     QStrList list = update->multipleSelection();
     if (!list.isEmpty())
-	{
+        {
             // Non-modal dialog
-	    MultiLogDialog *l = new MultiLogDialog();
-	    if (l->parseCvsLog(".", list))
+            MultiLogDialog *l = new MultiLogDialog();
+            if (l->parseCvsLog(".", list))
                 l->show();
             else
                 delete l;
-	}
+        }
 }
 #endif
 
@@ -938,7 +978,7 @@ void TopLevel::slotRemoveWatch()
 
 void TopLevel::addOrRemoveWatch(WatchDialog::ActionType action)
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -979,7 +1019,7 @@ void TopLevel::addOrRemoveWatch(WatchDialog::ActionType action)
 
 void TopLevel::slotShowWatchers()
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -997,7 +1037,7 @@ void TopLevel::slotShowWatchers()
 
 void TopLevel::slotEdit()
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -1015,7 +1055,7 @@ void TopLevel::slotEdit()
 
 void TopLevel::slotUnedit()
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -1034,7 +1074,7 @@ void TopLevel::slotUnedit()
 
 void TopLevel::slotLock()
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -1052,7 +1092,7 @@ void TopLevel::slotLock()
 
 void TopLevel::slotUnlock()
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -1070,7 +1110,7 @@ void TopLevel::slotUnlock()
 
 void TopLevel::slotShowEditors()
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -1102,11 +1142,11 @@ void TopLevel::slotMakePatch()
 
     QFile f(filename);
     if (!f.open(IO_WriteOnly))
-	{
-	    QMessageBox::information(this, "Cervisia",
-				     i18n("Could not open file for writing."));
-	    return;
-	}
+        {
+            QMessageBox::information(this, "Cervisia",
+                                     i18n("Could not open file for writing."));
+            return;
+        }
     QTextStream t(&f);
     QCString line;
     while (l.getOneLine(&line))
@@ -1154,7 +1194,7 @@ void TopLevel::importOrCheckout(CheckoutDialog::ActionType action)
                     if (!ignore.isEmpty())
                         {
                             cmdline += " -I ";
-                            cmdline += quote(ignore);
+                            cmdline += KShellProcess::quote(ignore);
                         }
                     cmdline += " -m ";
                     cmdline += (QString("\"") + "" + "\" "); // log message?
@@ -1197,7 +1237,7 @@ void TopLevel::slotDeleteTag()
 
 void TopLevel::createOrDeleteTag(TagDialog::ActionType action)
 {
-    QStrList list = update->multipleSelection();
+    QStringList list = update->multipleSelection();
     if (list.isEmpty())
         return;
 
@@ -1341,7 +1381,7 @@ void TopLevel::slotConfigure()
 void TopLevel::slotConfigureKeys()
 {
     if (KKeyDialog::configureKeys(accel))
-	updateAccel();
+        updateAccel();
 }
 
 
@@ -1359,7 +1399,7 @@ void TopLevel::slotCVSInfo()
 }
 
 
-void TopLevel::showJobStart(QString cmdline)
+void TopLevel::showJobStart(const QString &cmdline)
 {
     toolBar()->setItemEnabled(42, true);
     statusBar()->message(cmdline);
@@ -1375,7 +1415,7 @@ void TopLevel::slotJobFinished(bool)
 }
 
 
-void TopLevel::openSandbox(QString dirname)
+void TopLevel::openSandbox(const QString &dirname)
 {
     QFileInfo fi1(dirname);
     QString sandboxpath = fi1.absFilePath();
@@ -1385,14 +1425,14 @@ void TopLevel::openSandbox(QString dirname)
 
     QFileInfo fi2(sandboxpath + "/CVS");
     if (!fi2.exists() || !fi2.isDir())
-        {
-            QMessageBox::information(this, "Cervisia", i18n("This is not a CVS directory."));
-            return;
-        }
+    {
+        QMessageBox::information(this, "Cervisia", i18n("This is not a CVS directory."));
+        return;
+    }
 
-    recentFiles.insert(0, sandboxpath);
+    recentFiles.append(sandboxpath);
     while (recentFiles.count() > 10)
-        recentFiles.removeLast();
+        recentFiles.remove(recentFiles.fromLast());
 
     changelogstr = "";
     sandbox = sandboxpath;
@@ -1400,10 +1440,10 @@ void TopLevel::openSandbox(QString dirname)
 
     QFile f(sandbox + "/CVS/Root");
     if (f.open(IO_ReadOnly)) 
-	{ 
-	    QTextStream t(&f); 
-	    repository = t.readLine();
-	}
+    {
+        QTextStream t(&f);
+        repository = t.readLine();
+    }
     f.close();
     
 
@@ -1416,26 +1456,33 @@ void TopLevel::openSandbox(QString dirname)
     bool dostatus = config->readBoolEntry(repository.contains(":")?
                                           "StatusForRemoteRepos" : "StatusForLocalRepos", false);
     if (dostatus)
-        {
-            update->setSelected(update->firstChild(), true);
-            updateOrStatus(true, "");
-        }
+    {
+        update->setSelected(update->firstChild(), true);
+        updateOrStatus(true, "");
+    }
+
+    //load the recentCommits for this app from the KConfig app
+    config->setGroup( "CommitLogs" );
+    recentCommits = config->readListEntry( sandbox, COMMIT_SPLIT_CHAR );
+
 }
 
 
 void TopLevel::fillRecentPopup()
 {
     recent->clear();
-    QStrListIterator it(recentFiles);
     int n = 0;
-    for (; it.current(); ++it)
-        recent->insertItem(it.current(), n++);
+    for ( QStringList::Iterator it = recentFiles.begin();
+          it != recentFiles.end(); ++it )
+        {
+            recent->insertItem(*it, n++);            
+        }
 }
 
 
 void TopLevel::recentActivated(int n)
 {
-    openSandbox(recentFiles.at(n));
+    openSandbox( *(recentFiles.at(n)) );
 }
 
 
@@ -1456,7 +1503,7 @@ void TopLevel::setFilter()
     if (opt_hideUpToDate)
         filter = UpdateView::Filter(filter | UpdateView::NoUpToDate);
     if (opt_hideRemoved)
-	filter = UpdateView::Filter(filter | UpdateView::NoRemoved);
+        filter = UpdateView::Filter(filter | UpdateView::NoRemoved);
     update->setFilter(filter);
    
     settings->setItemChecked(settings_hidefiles, opt_hideFiles);
@@ -1465,14 +1512,14 @@ void TopLevel::setFilter()
 
     QString str;
     if (opt_hideFiles)
-	str = "F";
+        str = "F";
     else
-	{
-	    if (opt_hideUpToDate)
-		str += "N";
-	    if (opt_hideRemoved)
-		str += "R";
-	}
+        {
+            if (opt_hideUpToDate)
+                str += "N";
+            if (opt_hideRemoved)
+                str += "R";
+        }
     
     filterLabel->setText(str);
 }
@@ -1497,80 +1544,80 @@ void TopLevel::parseStatus(QString pathname, QStrList list)
 
     FILE *f = popen(command, "r");
     if (!f)
-	return;
+        return;
 
     state = Begin;
     while (fgets(buf, sizeof buf, f))
-	{
-	    QCString line = buf;
-	    chomp(&line);
-            //	    DEBUGOUT( "Line: " << line );
-	    switch (state)
-		{
-		case Begin:
-		    if (line.left(22) == "cvs status: Examining ")
-			dirpath = line.right(line.length()-22);
-		    state = File;
-                    //		    DEBUGOUT( "state = file" );
-		    break;
-		case File:
-		    if (line.length() > 32 &&
-			line.left(6) == "File: " &&
-			line.mid(24, 8) == "Status: ")
-			{
-			    name = line.mid(6, 18).stripWhiteSpace();
-			    if (dirpath != ".")
-				name.prepend("/").prepend(dirpath);
-			    statusstr = line.right(line.length()-32)
-				.stripWhiteSpace();
-			    state = FileSep;
-                            //			    DEBUGOUT( "state = FileSep" );
-			}
-		    break;
-		case FileSep:
-		    if (!line.isEmpty()) ; // Error
-		    state = WorkRev;
-                    //		    DEBUGOUT( "state = WorkRev" );
-		    break;
-		case WorkRev:
-		    if (line.left(21) == "   Working revision:\t")
-			{
-			    int pos;
-			    version = line.right(line.length()-21);
-			    if ( (pos = version.find(" ")) != -1 )
-				version.truncate(pos);
-			    state = FinalSep;
-                            //			    DEBUGOUT( "state = FinalSep" );
-			}
-		    break;
-		case FinalSep:
-		    if (line == "")
-			{
-                            //			    DEBUGOUT( "Adding: " << name <<
-                            //				      "Status: " << statusstr << "Version: " << version );
-			    UpdateView::Status status = UpdateView::Unknown;
-			    if (statusstr == "Up-to-date")
-				status = UpdateView::UpToDate;
-			    else if (statusstr == "Locally Modified")
-				status = UpdateView::LocallyModified;
-			    else if (statusstr == "Locally Added")
-				status = UpdateView::LocallyAdded;
-			    else if (statusstr == "Locally Removed")
-				status = UpdateView::LocallyRemoved;
-			    else if (statusstr == "Needs Checkout")
-				status = UpdateView::NeedsUpdate;
-			    else if (statusstr == "Needs Patch")
-				status = UpdateView::NeedsPatch;
-			    else if (statusstr == "Needs Merge")
-				status = UpdateView::NeedsMerge;
-			    else if (statusstr == "File had conflicts on merge")
-				status = UpdateView::Conflict;
-                            //			    update->addEntry(status, name /*, version*/);
-			    state = Begin;
-                            //			    DEBUGOUT( "state = Begin" );
-			}
-		}
-	}
+        {
+            QCString line = buf;
+            chomp(&line);
+            //            DEBUGOUT( "Line: " << line );
+            switch (state)
+                {
+                case Begin:
+                    if (line.left(22) == "cvs status: Examining ")
+                        dirpath = line.right(line.length()-22);
+                    state = File;
+                    //                    DEBUGOUT( "state = file" );
+                    break;
+                case File:
+                    if (line.length() > 32 &&
+                        line.left(6) == "File: " &&
+                        line.mid(24, 8) == "Status: ")
+                        {
+                            name = line.mid(6, 18).stripWhiteSpace();
+                            if (dirpath != ".")
+                                name.prepend("/").prepend(dirpath);
+                            statusstr = line.right(line.length()-32)
+                                .stripWhiteSpace();
+                            state = FileSep;
+                            //                            DEBUGOUT( "state = FileSep" );
+                        }
+                    break;
+                case FileSep:
+                    if (!line.isEmpty()) ; // Error
+                    state = WorkRev;
+                    //                    DEBUGOUT( "state = WorkRev" );
+                    break;
+                case WorkRev:
+                    if (line.left(21) == "   Working revision:\t")
+                        {
+                            int pos;
+                            version = line.right(line.length()-21);
+                            if ( (pos = version.find(" ")) != -1 )
+                                version.truncate(pos);
+                            state = FinalSep;
+                            //                            DEBUGOUT( "state = FinalSep" );
+                        }
+                    break;
+                case FinalSep:
+                    if (line == "")
+                        {
+                            //                            DEBUGOUT( "Adding: " << name <<
+                            //                                      "Status: " << statusstr << "Version: " << version );
+                            UpdateView::Status status = UpdateView::Unknown;
+                            if (statusstr == "Up-to-date")
+                                status = UpdateView::UpToDate;
+                            else if (statusstr == "Locally Modified")
+                                status = UpdateView::LocallyModified;
+                            else if (statusstr == "Locally Added")
+                                status = UpdateView::LocallyAdded;
+                            else if (statusstr == "Locally Removed")
+                                status = UpdateView::LocallyRemoved;
+                            else if (statusstr == "Needs Checkout")
+                                status = UpdateView::NeedsUpdate;
+                            else if (statusstr == "Needs Patch")
+                                status = UpdateView::NeedsPatch;
+                            else if (statusstr == "Needs Merge")
+                                status = UpdateView::NeedsMerge;
+                            else if (statusstr == "File had conflicts on merge")
+                                status = UpdateView::Conflict;
+                            //                            update->addEntry(status, name /*, version*/);
+                            state = Begin;
+                            //                            DEBUGOUT( "state = Begin" );
+                        }
+                }
+        }
     pclose(f);
 }
 #endif
@@ -1586,8 +1633,8 @@ bool TopLevel::queryExit()
             QString str;
             str.setNum(i);
             str.prepend("Sandbox");
-            config->writeEntry(str, recentFiles.at(0));
-            recentFiles.removeFirst();
+            config->writeEntry(str, *(recentFiles.at(0)));
+            recentFiles.remove(recentFiles.begin());
         }
     
     config->setGroup("Main window");
@@ -1625,7 +1672,7 @@ bool TopLevel::queryExit()
 }
 
 
-void TopLevel::restorePseudo(QString dirname)
+void TopLevel::restorePseudo(const QString &dirname)
 {
     KConfig *config = kapp->config();
 
