@@ -296,7 +296,8 @@ void UpdateDirItem::syncWithDirectory()
     // Do not use CvsDir here, because CVS/Entries may
     // contain files which are in .cvsignore (stupid
     // idea, but that's possible...)
-    QDir dir( dirPath(), QString::null, QDir::Name, QDir::Files|QDir::Hidden );
+    QDir dir( dirPath(), QString::null, QDir::Name,
+              QDir::Files|QDir::Hidden|QDir::NoSymLinks );
 
     const QFileInfoList *files = dir.exists()? dir.entryInfoList() : 0;
 
@@ -693,23 +694,26 @@ UpdateView::Filter UpdateView::filter() const
 
 bool UpdateView::hasSingleSelection()
 {
-    bool selfound = false;
-    QStack<QListViewItem> s;
+    return numSelections() == 1;
+}
 
+
+int UpdateView::numSelections()
+{
+    int num = 0;
+    
+    QStack<QListViewItem> s;
     for ( QListViewItem *item = firstChild(); item;
 	  item = item->nextSibling()? item->nextSibling() : s.pop() )
 	{
 	    if (item->firstChild())
-		    s.push(item->firstChild());
-
+                s.push(item->firstChild());
+            
 	    if (item->isSelected())
-		{
-		    if (selfound || item->isExpandable())
-			return false;
-		    selfound = true;
-		}
+                ++num;
 	}
-    return selfound;
+    
+    return num;
 }
 
 
@@ -732,34 +736,32 @@ void UpdateView::getSingleSelection(QString *filename, QString *revision)
 	}
 }
 
-
-QStrList UpdateView::multipleSelection()
+QStringList UpdateView::multipleSelection()
 {
-    QStrList res(true);
+    QStringList res;
     QStack<QListViewItem> s;
 
     for ( QListViewItem *item = firstChild(); item;
-	  item = item->nextSibling()? item->nextSibling() : s.pop() )
-	{
-	    if (item->firstChild())
-		s.push(item->firstChild());
+          item = item->nextSibling()? item->nextSibling() : s.pop() )
+        {
+            if (item->firstChild())
+                s.push(item->firstChild());
 
-	    if (item->isSelected())
-		{
-		    if (isDirItem(item))
-			{
-			    QString dirpath
-				= static_cast<UpdateDirItem*>(item)->dirPath();
-			    if (!dirpath.isEmpty())
-				dirpath.truncate(dirpath.length()-1);
+            if (item->isSelected())
+                {
+                    if (isDirItem(item))
+                        {
+                            QString dirpath = static_cast<UpdateDirItem*>(item)->dirPath();
+                            if (!dirpath.isEmpty())
+                                dirpath.truncate(dirpath.length()-1);
                             else
                                 dirpath = ".";
-			    res.append(dirpath);
-			}
-		    else
-			res.append( static_cast<UpdateViewItem*>(item)->filePath() );
-		}
-	}
+                            res.append(dirpath);
+                        }
+                    else
+                        res.append( static_cast<UpdateViewItem*>(item)->filePath() );
+                }
+        }
     return res;
 }
 
@@ -1034,7 +1036,7 @@ void UpdateView::processUpdateLine(QString str)
 }
 
 
-void UpdateView::updateItem(QString name, Status status, bool isdir)
+void UpdateView::updateItem(const QString &name, Status status, bool isdir)
 {
     // bla -> dirpath = "", filename = "bla"
     // bla/foo -> dirpath = "bla/", filename = "foo"
